window.gbUtils = {
    // XML to JSON converter
    // https://davidwalsh.name/convert-xml-json
    xmlToJson: function(xml) {
        // Create the return object
        var obj = {};

        if (xml.nodeType == 1) {
            // element
            // do attributes
            if (xml.attributes.length > 0) {
                obj['@attributes'] = {};
                for (var j = 0; j < xml.attributes.length; j++) {
                    var attribute = xml.attributes.item(j);
                    obj['@attributes'][attribute.nodeName] = attribute.nodeValue;
                }
            }
        } else if (xml.nodeType == 3 || xml.nodeType == 4) {
            // text
            obj = xml.nodeValue;
        }

        // do children
        if (xml.hasChildNodes()) {
            for (var i = 0; i < xml.childNodes.length; i++) {
                var item = xml.childNodes.item(i);
                var nodeName = item.nodeName;
                if (typeof obj[nodeName] == 'undefined') {
                    obj[nodeName] = window.gbUtils.xmlToJson(item);
                } else {
                    if (typeof obj[nodeName].push == 'undefined') {
                        var old = obj[nodeName];
                        obj[nodeName] = [];
                        obj[nodeName].push(old);
                    }
                    obj[nodeName].push(window.gbUtils.xmlToJson(item));
                }
            }
        }

        return obj;
    },
    randomString: function(length) {
        var result = '';
        var characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        var charactersLength = characters.length;

        for (var i = 0; i < length; i++) {
            result += characters.charAt(Math.floor(Math.random() * charactersLength));
        }

        return result;
    },
    request: function(options) {
        if (!options.proxyUrl) {
            options.proxyUrl = window.PROXY_URL;
        }

        if (!options.proxyUrlAccessToken) {
            options.proxyUrlAccessToken = window.PROXY_URL_ACCESS_TOKEN;
        }

        var ajaxOptions = {
            type: 'POST',
            contentType: 'application/json; charset=UTF-8',
            url: options.proxyUrl,
            headers: {}
        };

        if (options.proxyUrlAccessToken) {
            ajaxOptions.headers['Authorization'] = 'Bearer ' + options.proxyUrlAccessToken;
        }

        var timeout = options.timeout;

        if (!timeout || timeout > 90000) {
            timeout = 90000;
        }

        ajaxOptions.timeout = timeout;

        var data = {
            url: options.url,
            method: options.method
        };

        if (options.headers) {
            data.headers = options.headers;
        }

        if (options.dataValue) {
            if (!options.dataType) {
                options.dataType = 'application/json';
            }

            data.data = {
                type: options.dataType,
                value: options.dataValue
            };
        }

        ajaxOptions.data = JSON.stringify(data);

        if (options.promise === undefined) {
            options.promise = true;
        }

        if (options.promise) {
            return new Promise(function(resolve, reject) {
                ajaxOptions.success = function(data, textStatus, jqXHR) {
                    if (options.promiseExtendedResults) {
                        resolve({
                            data: data,
                            textStatus: textStatus,
                            jqXHR: jqXHR
                        });
                    } else {
                        resolve(data);
                    }
                };

                ajaxOptions.error = function(jqXHR, textStatus, errorThrown) {
                    if (options.promiseExtendedResults) {
                        reject({
                            jqXHR: jqXHR,
                            textStatus: textStatus,
                            errorThrown: errorThrown
                        });
                    } else {
                        var rejectMessages = [];

                        if (jqXHR) {
                            rejectMessages.push('status: ' + jqXHR.status);
                        }

                        if (textStatus) {
                            rejectMessages.push('textStatus: ' + textStatus);
                        }

                        if (errorThrown) {
                            rejectMessages.push('errorThrown: ' + errorThrown.toString());
                        }

                        if (jqXHR) {
                            if (jqXHR.responseJSON) {
                                rejectMessages.push('response: ' + jqXHR.responseJSON);
                            } else if (jqXHR.responseText) {
                                rejectMessages.push('response: ' + jqXHR.responseText);
                            }
                        }

                        reject(rejectMessages.join(', '));
                    }
                };

                $.ajax(ajaxOptions);
            });
        } else {
            if (options.complete) {
                ajaxOptions.complete = function(jqXHR, textStatus) {
                    options.complete(jqXHR, textStatus);
                };
            }

            if (options.success) {
                ajaxOptions.success = function(data, textStatus, jqXHR) {
                    options.success(data, textStatus, jqXHR);
                };
            }

            if (options.error) {
                ajaxOptions.error = function(jqXHR, textStatus, errorThrown) {
                    options.error(jqXHR, textStatus, errorThrown);
                };
            }

            $.ajax(ajaxOptions);
        }
    }
};