function widgetScriptingResolveListener(widget, listener) {
	var result = Promise.resolve();

	if (widget.listeners && widget.listeners[listener + 'Listener']) {
		result = Promise.resolve(widget.listeners[listener + 'Listener'](widget));
	}

	return result.catch(function (error) {
		console.error('Widget ' + widget.id + ' caused an error on listener ' + listener, error);
	});
}

function widgetScriptingResolveListenerPage(widget, listener) {
	var result = Promise.resolve();

	if (widget.listeners && widget.listeners.page && widget.listeners.page[listener.substr(5) + 'Listener']) {
		result = Promise.resolve(widget.listeners.page[listener.substr(5) + 'Listener'](widget));
	}

	return result.catch(function (error) {
		console.error('Widget ' + widget.id + ' caused an error on page listener ' + listener, error);
	});
}

function widgetScriptingAddListenersToWidget(widget) {
	widget.listeners = {
		onLoadListener: null,
		onLoadedListener: null,
		onShowListener: null,
		onHideListener: null,
		onUnloadListener: null
	};
	widget.listeners.page = {
		onLoadListener: null,
		onLoadedListener: null,
		onShowListener: null,
		onHideListener: null,
		onUnloadListener: null
	};

	widget.onLoad = function (listenerFunction) {
		widget.listeners.onLoadListener = listenerFunction;
	};
    widget.onLoaded = function (listenerFunction) {
		widget.listeners.onLoadedListener = listenerFunction;
	};
	widget.onShow = function (listenerFunction) {
		widget.listeners.onShowListener = listenerFunction;
	};
	widget.onHide = function (listenerFunction) {
		widget.listeners.onHideListener = listenerFunction;
	};
	widget.onUnload = function (listenerFunction) {
		widget.listeners.onUnloadListener = listenerFunction;
	};
	widget.page = {};
    widget.page.onLoaded = function (listenerFunction) {
		widget.listeners.page.onLoadedListener = listenerFunction;
	};
	widget.page.onLoad = function (listenerFunction) {
		widget.listeners.page.onLoadListener = listenerFunction;
	};
	widget.page.onShow = function (listenerFunction) {
		widget.listeners.page.onShowListener = listenerFunction;
	};
	widget.page.onHide = function (listenerFunction) {
		widget.listeners.page.onHideListener = listenerFunction;
	};
	widget.page.onUnload = function (listenerFunction) {
		widget.listeners.page.onUnloadListener = listenerFunction;
	};

	widget.events = {};

	widget.events.durationDeterminationHeartbeat = function (calculatedEndTime) {
		widgetScriptingDurationDeterminationMessage(widget.id, 0, { calculatedEndTime: calculatedEndTime });
	};
	widget.events.durationDeterminationReady = function () {
		widgetScriptingDurationDeterminationMessage(widget.id, 10, {});
	};
}

function widgetScriptingDurationDeterminationMessage(id, action, parameters) {
	var message = {
		id: id,
		type: 'durationDetermination',
		durationDeterminationType: action,
		durationDeterminationParameters: parameters
	};

	windowMessage(message);
}

function widgetScriptingOnActionMessage(id, action, parameters) {
	var message = {
		id: id,
		type: 'action',
		actionType: action,
		actionParameters: parameters
	};

	windowMessage(message);
}

function widgetScriptingGetWidgetPages(id) {
	var widgetElement = document.getElementById(id);

	return widgetElement.getElementsByTagName('page');
}

function widgetScriptingGetWidgetPageContainer(id) {
	var widgetElement = document.getElementById(id);

	return widgetElement.getElementsByTagName('page-container')[0];
}

function widgetScriptingEventExecutor(listener, widget) {
	switch (listener) {
		case 'onLoad':
			var scriptTag = document.createElement('script');
			scriptTag.type = 'text/javascript';
			scriptTag.id = widget.id + '_script';

			scriptTag.innerHTML = 'widgetScriptingMethods.' + widget.id + ' = function (widget) {' + widget.javascriptSource + '}';

			document.head.appendChild(scriptTag);

			window.widgetScriptingMethods[widget.id](widget);

			return widgetScriptingResolveListener(widget, listener);
        case 'onLoaded':
            return widgetScriptingResolveListener(widget, listener);
		case 'onShow':
			return widgetScriptingResolveListener(widget, listener);
		case 'onHide':
			return widgetScriptingResolveListener(widget, listener);
		case 'onUnload':
			return widgetScriptingResolveListenerPage(widget, listener).then(function () {
				var styleTag = document.getElementById(widget.id + '_style');

				if (styleTag) {
					if (styleTag.parentNode) {
						styleTag.parentNode.removeChild(styleTag);
					} else {
						delete styleTag;
					}
				}

				window.widgetScriptingMethods[widget.id] = null;

				delete window.widgetScriptingMethods[widget.id];

				var scriptTag = document.getElementById(widget.id + '_script');

				if (scriptTag) {
					if (scriptTag.parentNode) {
						scriptTag.parentNode.removeChild(scriptTag);
					} else {
						delete scriptTag;
					}
				}

				document.getElementById(widget.id).children[0].innerHTML = '';
				document.getElementById(widget.id).children[0].removeAttribute('data-initialized');
			});
		case 'page.onLoad':
			return widgetScriptingResolveListenerPage(widget, listener).then(function () {
				doT.templateSettings.varname = 'widget';

				var renderingElement = document.getElementById(widget.id).children[0];

				if (!renderingElement.dataset || !renderingElement.dataset.initialized) {
					var cssTemplate = doT.template(widget.cssSource);
					var cssTemplateCompiled = cssTemplate(widget);

					var styleTag = document.createElement('style');
					styleTag.type = 'text/css';
					styleTag.id = widget.id + '_style';
					styleTag.innerText = cssTemplateCompiled;

					document.head.appendChild(styleTag);

					var htmlTemplate = doT.template(widget.htmlSource);
					var htmlTemplateCompiled = htmlTemplate(widget);

					var start = htmlTemplateCompiled.indexOf('<repeating>');
					var end = htmlTemplateCompiled.indexOf('</repeating>');

					var beforeRepeat = '';
					var afterRepeat = '';
					var insideRepeat = '';

					if (start > 0 && end > 0) {
						beforeRepeat = htmlTemplateCompiled.substring(0, start);
						insideRepeat = htmlTemplateCompiled.substring(start + 11, end);
						afterRepeat = htmlTemplateCompiled.substring(end + 12, htmlTemplateCompiled.length);
					} else {
						// Because there is no repeating element, the entire HTML is the page
						insideRepeat = htmlTemplateCompiled;
					}

					// Surround the repeating element with a page element and a page-container element
					htmlTemplateCompiled = beforeRepeat + '<page-container><page>' + insideRepeat + '</page></page-container>' + afterRepeat;

					renderingElement.innerHTML = htmlTemplateCompiled;
					renderingElement.setAttribute('data-initialized', true);
				} else {
					var existingPageElements = widgetScriptingGetWidgetPages(widget.id);

					// Remove all existing page elements, except the first one.
					if (existingPageElements.length >= 2) {
						for (var i = 0; i < existingPageElements.length - 1; i++) {
							existingPageElements[i].remove();
						}
					}

					var element = document.createElement('page');
					var html = widget.htmlSourcePage;

					doT.templateSettings.varname = 'widget';

					var template = doT.template(html.replace(/[\\n\\r]/g, ''));

					var compiledHtml = template(widget);

					element.innerHTML = compiledHtml;
					element.style.display = 'none';

					widgetScriptingGetWidgetPageContainer(widget.id).appendChild(element);
				}
            }); 
        case 'page.onLoaded':
            return widgetScriptingResolveListenerPage(widget, listener);
		case 'page.onShow':
			return widgetScriptingResolveListenerPage(widget, listener);
		case 'page.onHide':
			return widgetScriptingResolveListenerPage(widget, listener);
		case 'page.onUnload':
			return widgetScriptingResolveListenerPage(widget, listener);
	}
}
